@@ -4,7 +4,7 @@
     created              : Wed Apr 26 20:05:12 CEST 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: driverconfig.cpp,v 1.7.2.6 2011/12/29 16:14:20 berniw Exp $
+    version              : $Id: driverconfig.cpp,v 1.7.2.8 2012/06/10 01:38:23 berniw Exp $
 
  ***************************************************************************/
 
@@ -216,7 +216,7 @@ GenCarsInfo(void)
 			GF_TAILQ_INSERT_TAIL(&CatsInfoList, curCat, link);
 		} while (curFile != files);
 	}
-	GfDirFreeList(files, NULL, true, true);
+	GfDirFreeList(files, NULL, true, false);
 	
 	files = GfDirGetList("cars");
 	curFile = files;
@@ -244,9 +244,10 @@ GenCarsInfo(void)
 			}
 			curCar->cat = curCat;
 			GF_TAILQ_INSERT_TAIL(&(curCat->CarsInfoList), curCar, link);
+			GfParmReleaseHandle(carparam);
 		} while (curFile != files);
 	}
-	GfDirFreeList(files, NULL, true, true);
+	GfDirFreeList(files, NULL, true, false);
 	
 	/* Remove the empty categories */
 	curCat = GF_TAILQ_FIRST(&CatsInfoList);
