@@ -2,9 +2,9 @@
 
     file        : raceengine.cpp
     created     : Sat Nov 23 09:05:23 CET 2002
-    copyright   : (C) 2002 by Eric Espié 
+    copyright   : (C) 2002-2014 by Eric Espie, Bernhard Wymann 
     email       : eric.espie@torcs.org 
-    version     : $Id: raceengine.cpp,v 1.19.2.6 2012/02/14 23:42:20 berniw Exp $
+    version     : $Id: raceengine.cpp,v 1.19.2.23 2014/08/05 23:05:06 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: raceengine.cpp,v 1.19.2.6 2012/02/14 23:42:20 berniw Exp $
+    @version	$Id: raceengine.cpp,v 1.19.2.23 2014/08/05 23:05:06 berniw Exp $
 */
 
 #include <stdlib.h>
@@ -57,8 +57,16 @@ ReUpdtPitTime(tCarElt *car)
 
 	switch (car->_pitStopType) {
 		case RM_PIT_REPAIR:
-			info->totalPitTime = 2.0f + fabs((double)(car->_pitFuel)) / 8.0f + (tdble)(fabs((double)(car->_pitRepair))) * 0.007f;
+			info->totalPitTime = ReInfo->raceRules.pitstopBaseTime + fabs((double)(car->_pitFuel)) / ReInfo->raceRules.refuelFuelFlow + (tdble)(fabs((double)(car->_pitRepair))) * ReInfo->raceRules.damageRepairFactor + car->_penaltyTime;
+			if (ReInfo->s->raceInfo.type == RM_TYPE_PRACTICE || ReInfo->s->raceInfo.type == RM_TYPE_QUALIF) { 
+				// Ensure that the right min/max values are in the setup structure (could have been modified by the robot))
+				RtInitCarPitSetup(car->_carHandle, &(car->pitcmd.setup), true);
+			} else {
+				// In case of the race no modifications are allowed, so completely reload the structure
+				RtInitCarPitSetup(car->_carHandle, &(car->pitcmd.setup), false);
+			}
 			car->_scheduledEventTime = s->currentTime + info->totalPitTime;
+			car->_penaltyTime = 0.0f;
 			ReInfo->_reSimItf.reconfig(car);
 			for (i=0; i<4; i++) {
 				car->_tyreCondition(i) = 1.01;
@@ -68,8 +76,9 @@ ReUpdtPitTime(tCarElt *car)
 			}
 			break;
 		case RM_PIT_STOPANDGO:
-			info->totalPitTime = 0.0;
-			car->_scheduledEventTime = s->currentTime;
+			info->totalPitTime = car->_penaltyTime;
+			car->_scheduledEventTime = s->currentTime + info->totalPitTime;
+			car->_penaltyTime = 0.0f;
 			break;
 	}
 }
@@ -81,7 +90,6 @@ ReUpdtPitCmd(void *pvcar)
 	tCarElt *car = (tCarElt*)pvcar;
 
 	ReUpdtPitTime(car);
-	//ReStart(); /* resynchro */
 	GfuiScreenActivate(ReInfo->_reGameScreen);
 }
 
@@ -97,7 +105,7 @@ ReRaceMsgUpdate(void)
 }
 
 static void
-ReRaceMsgSet(char *msg, double life)
+ReRaceMsgSet(const char *msg, double life)
 {
 	if ((ReInfo->_displayMode != RM_DISP_MODE_NONE) && (ReInfo->_displayMode != RM_DISP_MODE_CONSOLE)) {
 		ReSetRaceMsg(msg);
@@ -140,6 +148,7 @@ ReManage(tCarElt *car)
 	
 	if (car->_speed_x < info->botSpd) {
 		info->botSpd = car->_speed_x;
+		car->_currentMinSpeedForLap = car->_speed_x;
 	}
 	
 	// Pitstop.
@@ -219,7 +228,7 @@ ReManage(tCarElt *car)
 					if (car->robot->rbPitCmd(car->robot->index, car, s) == ROB_PIT_MENU) {
 						// the pit cmd is modified by menu.
 						ReStop();
-						RmPitMenuStart(car, (void*)car, ReUpdtPitCmd);
+						RmPitMenuStart(car, ReInfo, (void*)car, ReUpdtPitCmd);
 					} else {
 						ReUpdtPitTime(car);
 					}
@@ -242,8 +251,13 @@ ReManage(tCarElt *car)
 							car->_deltaBestLapTime = car->_lastLapTime - car->_bestLapTime;
 						}
 						if ((car->_lastLapTime < car->_bestLapTime) || (car->_bestLapTime == 0)) {
-							car->_bestLapTime = car->_lastLapTime;
+							if (car->_commitBestLapTime) {
+								car->_bestLapTime = car->_lastLapTime;
+							}
 						}
+						
+						car->_commitBestLapTime = true;
+						
 						if (car->_pos != 1) {
 							car->_timeBehindLeader = car->_curTime - s->cars[0]->_curTime;
 							car->_lapsBehindLeader = s->cars[0]->_laps - car->_laps;
@@ -263,15 +277,14 @@ ReManage(tCarElt *car)
 							case RM_TYPE_PRACTICE:
 								if (ReInfo->_displayMode == RM_DISP_MODE_NONE) {
 									ReInfo->_refreshDisplay = 1;
-									char *t1, *t2;
-									t1 = GfTime2Str(car->_lastLapTime, 0);
-									t2 = GfTime2Str(car->_bestLapTime, 0);
+									const int TIMEFMTSIZE=256;
+									char t1[TIMEFMTSIZE], t2[TIMEFMTSIZE];
+									GfTime2Str(t1, TIMEFMTSIZE, car->_lastLapTime, 0);
+									GfTime2Str(t2, TIMEFMTSIZE, car->_bestLapTime, 0);
 									snprintf(buf, BUFSIZE, "lap: %02d   time: %s  best: %s  top spd: %.2f    min spd: %.2f    damage: %d",
 										car->_laps - 1, t1, t2,
 										info->topSpd * 3.6, info->botSpd * 3.6, car->_dammage);
 									ReResScreenAddText(buf);
-									free(t1);
-									free(t2);
 								}
 								/* save the lap result */
 								ReSavePracticeLap(car);
@@ -291,6 +304,7 @@ ReManage(tCarElt *car)
 			
 					info->topSpd = car->_speed_x;
 					info->botSpd = car->_speed_x;
+					car->_currentMinSpeedForLap = car->_speed_x;
 					if ((car->_remainingLaps < 0) || (s->_raceState == RM_RACE_FINISHING)) {
 						car->_state |= RM_CAR_STATE_FINISH;
 						s->_raceState = RM_RACE_FINISHING;
@@ -346,54 +360,56 @@ ReManage(tCarElt *car)
 	car->_distRaced = (car->_laps - (info->lapFlag + 1)) * ReInfo->track->length + car->_distFromStartLine;
 }
 
-static void 
-ReSortCars(void)
-{
-    int		i,j;
-    tCarElt	*car;
-    int		allfinish;
-    tSituation	*s = ReInfo->s;
 
-    if ((s->cars[0]->_state & RM_CAR_STATE_FINISH) == 0) {
-	allfinish = 0;
-    } else {
-	allfinish = 1;
-    }
-    
-    for (i = 1; i < s->_ncars; i++) {
-	j = i;
-	while (j > 0) {
-	    if ((s->cars[j]->_state & RM_CAR_STATE_FINISH) == 0) {
+static void ReSortCars(void)
+{
+	int i, j;
+	tCarElt	*car;
+	int	allfinish;
+	tSituation *s = ReInfo->s;
+
+	if ((s->cars[0]->_state & RM_CAR_STATE_FINISH) == 0) {
 		allfinish = 0;
-		if (s->cars[j]->_distRaced > s->cars[j-1]->_distRaced) {
-		    car = s->cars[j];
-		    s->cars[j] = s->cars[j-1];
-		    s->cars[j-1] = car;
-		    s->cars[j]->_pos = j+1;
-		    s->cars[j-1]->_pos = j;
-		    j--;
-		    continue;
+	} else {
+		allfinish = 1;
+	}
+
+	for (i = 1; i < s->_ncars; i++) {
+		j = i;
+		while (j > 0) {
+			if ((s->cars[j]->_state & RM_CAR_STATE_FINISH) == 0) {
+				allfinish = 0;
+				if (s->cars[j]->_distRaced > s->cars[j-1]->_distRaced) {
+					car = s->cars[j];
+					s->cars[j] = s->cars[j-1];
+					s->cars[j-1] = car;
+					s->cars[j]->_pos = j+1;
+					s->cars[j-1]->_pos = j;
+					j--;
+					continue;
+				}
+			}
+			j = 0;
 		}
-	    }
-	    j = 0;
+	}
+
+	if (allfinish) {
+		ReInfo->s->_raceState = RM_RACE_ENDED;
 	}
-    }
-    if (allfinish) {
-	ReInfo->s->_raceState = RM_RACE_ENDED;
-    }
 }
 
+
 /* Compute the race rules and penalties */
 static void
 ReRaceRules(tCarElt *car)
 {
-    tCarPenalty		*penalty;
-    tTrack		*track = ReInfo->track;
-    tRmCarRules		*rules = &(ReInfo->rules[car->index]);
-    tTrackSeg		*seg = RtTrackGetSeg(&(car->_trkPos));
-    tReCarInfo		*info = &(ReInfo->_reCarInfo[car->index]);
-    tTrackSeg		*prevSeg = RtTrackGetSeg(&(info->prevTrkPos));
-    static float	color[] = {0.0, 0.0, 1.0, 1.0};
+	tCarPenalty *penalty;
+	tTrack *track = ReInfo->track;
+	tRmCarRules *rules = &(ReInfo->rules[car->index]);
+	tTrackSeg *seg = RtTrackGetSeg(&(car->_trkPos));
+	tReCarInfo *info = &(ReInfo->_reCarInfo[car->index]);
+	tTrackSeg *prevSeg = RtTrackGetSeg(&(info->prevTrkPos));
+	static float color[] = {0.0, 0.0, 1.0, 1.0};
 
 	// DNF cars which need too much time for the current lap, this is mainly to avoid
 	// that a "hanging" driver can stop the quali from finishing.
@@ -408,6 +424,76 @@ ReRaceRules(tCarElt *car)
 	    return;
 	}
 
+	const int BUFSIZE = 1024;
+	char buf[BUFSIZE];
+
+	// Ignore some rules after the car has finished the race
+	if ((car->pub.state & RM_CAR_STATE_FINISH) == 0) {
+
+		// If a car hits the track wall the lap time is invalidated, because of tracks where this behaviour allows much faster laps (e.g. alpine-2)
+		// Invalidation and message is just shown on the first hit
+		if (ReInfo->raceRules.enabled & RmRaceRules::WALL_HIT_TIME_INVALIDATE) {
+			if (car->_commitBestLapTime && (car->priv.simcollision & SEM_COLLISION_XYSCENE)) {
+				car->_commitBestLapTime = false;
+				if (ReInfo->s->_raceType != RM_TYPE_RACE) {
+					ReRaceMsgSet("Hit wall, laptime invalidated", 5);
+				}
+			}
+		}
+			
+		// If the car cuts a corner the lap time is invalidated. Cutting a corner means: the center of gravity is more than 0.7 times the car width
+		// away from the main track segment on the inside of a turn. The rule does not apply on the outside and on straights, pit entry and exit
+		// count as well as track.
+		tTrackSeg *mainseg = car->_trkPos.seg;
+		bool pit = false;
+		tTrackPitInfo pitInfo = track->pits;
+		tdble toborder = 0.0f;
+		tdble minradius = 1.0f;
+
+		if (mainseg->type != TR_STR) {
+			if (track->pits.type == TR_PIT_ON_TRACK_SIDE) {
+				if (pitInfo.pitEntry->id < pitInfo.pitExit->id) {
+					if ((mainseg->id >= pitInfo.pitEntry->id) && (mainseg->id <= pitInfo.pitExit->id)) {
+						pit = true;
+					}
+				} else {
+					if ((mainseg->id >= pitInfo.pitEntry->id) || (mainseg->id <= pitInfo.pitExit->id)) {
+						pit = true;
+					}
+				}
+			}
+
+			if (mainseg->type == TR_LFT) {
+				if (!(pit && (pitInfo.side == TR_LFT))) {
+					toborder = car->_trkPos.toLeft;
+					minradius = mainseg->radiusl;
+				}
+			} else if (mainseg->type == TR_RGT) {
+				if (!(pit && (pitInfo.side == TR_RGT))) {
+					toborder = car->_trkPos.toRight;
+					minradius = mainseg->radiusr;
+				}
+			}
+		}
+
+		tdble cuttinglimit = car->_dimension_y*0.7f;
+		if (toborder < -cuttinglimit) {
+			if (ReInfo->raceRules.enabled & RmRaceRules::CORNER_CUTTING_TIME_INVALIDATE) {
+				if (ReInfo->s->_raceType != RM_TYPE_RACE && car->_commitBestLapTime) {
+					ReRaceMsgSet("Cut corner, laptime invalidated", 5);
+				}
+				car->_commitBestLapTime = false;
+			}
+			if (ReInfo->s->_raceType == RM_TYPE_RACE && ReInfo->raceRules.enabled & RmRaceRules::CORNER_CUTTING_TIME_PENALTY) {
+				// In race, apply additionally corner cutting time penalty
+				minradius -= cuttinglimit;
+				if (minradius > 1.0f) {
+					car->_penaltyTime += car->pub.speed*RCM_MAX_DT_SIMU*(-toborder-cuttinglimit)/minradius;
+				}			
+			}
+		}
+	}
+
 	if (car->_skillLevel < 3) {
 		/* only for the pros */
 		return;
@@ -436,9 +522,6 @@ ReRaceRules(tCarElt *car)
 		memcpy(car->ctrl.msgColor, color, sizeof(car->ctrl.msgColor));
 	}
     
-	const int BUFSIZE = 1024;
-	char buf[BUFSIZE];
-
 	if (prevSeg->raceInfo & TR_PITSTART) {
 		/* just entered the pit lane */
 		if (seg->raceInfo & TR_PIT) {
@@ -593,6 +676,7 @@ ReStart(void)
 void
 ReStop(void)
 {
+	ReInfo->_reGraphicItf.muteformenu();
     ReInfo->_reRunning = 0;
 }
 
@@ -628,19 +712,27 @@ ReUpdate(void)
 	double t;
 	tRmMovieCapture	*capture;
 	int mode = RM_ASYNC;
-
+	int i;
+	const int MAXSTEPS = 2000;
+	
 	START_PROFILE("ReUpdate");
 	ReInfo->_refreshDisplay = 0;
 	switch (ReInfo->_displayMode) {
 		case RM_DISP_MODE_NORMAL:
 			t = GfTimeClock();
-
+			
+			i = 0;
 			START_PROFILE("ReOneStep*");
-			while (ReInfo->_reRunning && ((t - ReInfo->_reCurTime) > RCM_MAX_DT_SIMU)) {
+			while ((ReInfo->_reRunning && ((t - ReInfo->_reCurTime) > RCM_MAX_DT_SIMU)) && MAXSTEPS > i++) {
 				ReOneStep(RCM_MAX_DT_SIMU);
 			}
 			STOP_PROFILE("ReOneStep*");
 
+			if (i > MAXSTEPS) {
+				// Cannot keep up with time warp, reset time to avoid lag when running slower again
+				ReInfo->_reCurTime = GfTimeClock();
+			}
+			
 			GfuiDisplay();
 			ReInfo->_reGraphicItf.refresh(ReInfo->s);
 			glutPostRedisplay();	/* Callback -> reDisplay */
@@ -699,8 +791,8 @@ ReTimeMod (void *vcmd)
 			break;
 		case 1:
 			ReInfo->_reTimeMult *= 0.5;
-			if (ReInfo->_reTimeMult < 0.25) {
-				ReInfo->_reTimeMult = 0.25;
+			if (ReInfo->_reTimeMult < 1.0f/128.0f) {
+				ReInfo->_reTimeMult = 1.0f/128.0f;
 			}
 			break;
 		case 2:
