@@ -2,9 +2,9 @@
 
     file                 : results.cpp
     created              : Fri Apr 14 22:36:36 CEST 2000
-    copyright            : (C) 2000 by Eric Espie
+    copyright            : (C) 2000-2014 by Eric Espie, Bernhard Wymann
     email                : torcs@free.fr
-    version              : $Id: results.cpp,v 1.6.2.5 2012/02/17 13:03:26 berniw Exp $
+    version              : $Id: results.cpp,v 1.6.2.10 2014/05/23 08:38:31 berniw Exp $
 
  ***************************************************************************/
 
@@ -18,10 +18,9 @@
  ***************************************************************************/
 
 /** @file
-    		This is a set of tools useful for race managers to display results.
-    @ingroup	racemantools
-    @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: results.cpp,v 1.6.2.5 2012/02/17 13:03:26 berniw Exp $
+    Display results.
+    @author Bernhard Wymann, Eric Espie
+    @version $Id: results.cpp,v 1.6.2.10 2014/05/23 08:38:31 berniw Exp $
 */
 
 #include <stdlib.h>
@@ -48,26 +47,25 @@ static void rmShowStandings(void *prevHdle, tRmInfo *info, int start);
 
 typedef struct
 {
-    void	*prevHdle;
-    tRmInfo	*info;
-    int		start;
+	void *prevHdle;
+	tRmInfo *info;
+	int start;
 } tRaceCall;
 
-tRaceCall	RmNextRace;
-tRaceCall	RmPrevRace;
+tRaceCall RmNextRace;
+tRaceCall RmPrevRace;
 
 
-static void
-rmSaveRes(void *vInfo)
+static void rmSaveRes(void *vInfo)
 {
 	tRmInfo *info = (tRmInfo *)vInfo;
+	GfParmCreateDirectory(0, info->results);
 	GfParmWriteFile(0, info->results, "Results");
 	GfuiVisibilitySet(rmScrHdle, rmSaveId, GFUI_INVISIBLE);
 }
 
 
-static void
-rmChgPracticeScreen(void *vprc)
+static void rmChgPracticeScreen(void *vprc)
 {
 	void *prevScr = rmScrHdle;
 	tRaceCall *prc = (tRaceCall*)vprc;
@@ -77,8 +75,7 @@ rmChgPracticeScreen(void *vprc)
 }
 
 
-static void
-rmPracticeResults(void *prevHdle, tRmInfo *info, int start)
+static void rmPracticeResults(void *prevHdle, tRmInfo *info, int start)
 {
 	void *results = info->results;
 	const char *race = info->_reRaceName;
@@ -89,7 +86,8 @@ rmPracticeResults(void *prevHdle, tRmInfo *info, int start)
 	const int BUFSIZE = 1024;
 	char buf[BUFSIZE];
 	char path[BUFSIZE];
-	char *str;
+	const int TIMEFMTSIZE = 256;
+	char timefmt[TIMEFMTSIZE];
 	float fgcolor[4] = {1.0, 0.0, 1.0, 1.0};
 	int totLaps;
 
@@ -130,14 +128,12 @@ rmPracticeResults(void *prevHdle, tRmInfo *info, int start)
 		GfuiLabelCreate(rmScrHdle, buf, GFUI_FONT_MEDIUM_C, x1, y, GFUI_ALIGN_HC_VB, 0);
 
 		/* Time */
-		str = GfTime2Str(GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0), 0);;
-		GfuiLabelCreate(rmScrHdle, str, GFUI_FONT_MEDIUM_C, x2, y, GFUI_ALIGN_HL_VB, 0);
-		free(str);
+		GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0), 0);
+		GfuiLabelCreate(rmScrHdle, timefmt, GFUI_FONT_MEDIUM_C, x2, y, GFUI_ALIGN_HL_VB, 0);
 
 		/* Best Lap Time */
-		str = GfTime2Str(GfParmGetNum(results, path, RE_ATTR_BEST_LAP_TIME, NULL, 0), 0);;
-		GfuiLabelCreate(rmScrHdle, str, GFUI_FONT_MEDIUM_C, x3, y, GFUI_ALIGN_HL_VB, 0);
-		free(str);
+		GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_BEST_LAP_TIME, NULL, 0), 0);
+		GfuiLabelCreate(rmScrHdle, timefmt, GFUI_FONT_MEDIUM_C, x3, y, GFUI_ALIGN_HL_VB, 0);
 
 		/* Top Spd */
 		snprintf(buf, BUFSIZE, "%d", (int)(GfParmGetNum(results, path, RE_ATTR_TOP_SPEED, NULL, 0) * 3.6));
@@ -180,20 +176,6 @@ rmPracticeResults(void *prevHdle, tRmInfo *info, int start)
 			(tfuiCallback)NULL,
 			(tfuiCallback)NULL);
 
-/*     rmSaveId = GfuiButtonCreate(rmScrHdle, */
-/* 				"Save", */
-/* 				GFUI_FONT_LARGE, */
-/* 				430, */
-/* 				40, */
-/* 				150, */
-/* 				GFUI_ALIGN_HC_VB, */
-/* 				0, */
-/* 				info, */
-/* 				rmSaveRes, */
-/* 				NULL, */
-/* 				(tfuiCallback)NULL, */
-/* 				(tfuiCallback)NULL); */
-
 	if (i < totLaps) {
 		RmNextRace.prevHdle = prevHdle;
 		RmNextRace.info     = info;
@@ -214,8 +196,7 @@ rmPracticeResults(void *prevHdle, tRmInfo *info, int start)
 }
 
 
-static void
-rmChgRaceScreen(void *vprc)
+static void rmChgRaceScreen(void *vprc)
 {
 	void *prevScr = rmScrHdle;
 	tRaceCall *prc = (tRaceCall*)vprc;
@@ -225,19 +206,19 @@ rmChgRaceScreen(void *vprc)
 }
 
 
-static void
-rmRaceResults(void *prevHdle, tRmInfo *info, int start)
+static void rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 {
 	void *results = info->results;
 	const char *race = info->_reRaceName;
 	int i;
-	int x1, x2, x3, x4, x5, x6, x7, x8;
+	int x1, x2, x3, x4, x5, x6, x7, x8, x9;
 	int dlap;
 	int y;
 	const int BUFSIZE = 1024;
 	char buf[BUFSIZE];
 	char path[BUFSIZE];
-	char *str;
+	const int TIMEFMTSIZE = 256;
+	char timefmt[TIMEFMTSIZE];
 	float fgcolor[4] = {1.0, 0.0, 1.0, 1.0};
 	int laps, totLaps;
 	tdble refTime;
@@ -253,12 +234,13 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 	
 	x1 = 30;
 	x2 = 60;
-	x3 = 240;
+	x3 = 260;
 	x4 = 330;
 	x5 = 360;
 	x6 = 420;
-	x7 = 510;
-	x8 = 595;
+	x7 = 490;
+	x8 = 545;
+	x9 = 630;
 	
 	y = 400;
 	GfuiLabelCreateEx(rmScrHdle, "Rank",      fgcolor, GFUI_FONT_MEDIUM_C, x1, y, GFUI_ALIGN_HC_VB, 0);
@@ -267,8 +249,9 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 	GfuiLabelCreateEx(rmScrHdle, "Best",      fgcolor, GFUI_FONT_MEDIUM_C, x4, y, GFUI_ALIGN_HR_VB, 0);
 	GfuiLabelCreateEx(rmScrHdle, "Laps",      fgcolor, GFUI_FONT_MEDIUM_C, x5, y, GFUI_ALIGN_HC_VB, 0);
 	GfuiLabelCreateEx(rmScrHdle, "Top Spd",   fgcolor, GFUI_FONT_MEDIUM_C, x6, y, GFUI_ALIGN_HC_VB, 0);
-	GfuiLabelCreateEx(rmScrHdle, "Damages",  fgcolor, GFUI_FONT_MEDIUM_C, x7, y, GFUI_ALIGN_HC_VB, 0);
-	GfuiLabelCreateEx(rmScrHdle, "Pit Stops", fgcolor, GFUI_FONT_MEDIUM_C, x8, y, GFUI_ALIGN_HC_VB, 0);
+	GfuiLabelCreateEx(rmScrHdle, "Damage",    fgcolor, GFUI_FONT_MEDIUM_C, x7, y, GFUI_ALIGN_HC_VB, 0);
+	GfuiLabelCreateEx(rmScrHdle, "Pit",       fgcolor, GFUI_FONT_MEDIUM_C, x8, y, GFUI_ALIGN_HC_VB, 0);
+	GfuiLabelCreateEx(rmScrHdle, "Penalty",   fgcolor, GFUI_FONT_MEDIUM_C, x9, y, GFUI_ALIGN_HR_VB, 0);	
 	y -= 20;
 	
 	snprintf(path, BUFSIZE, "%s/%s/%s", info->track->name, RE_SECT_RESULTS, race);
@@ -290,12 +273,11 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 
 		if (laps == totLaps) {
 			if (i == 0) {
-				str = GfTime2Str(GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0), 0);
+				GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0), 0);
 			} else {
-				str = GfTime2Str(GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0) - refTime, 1);
+				GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0) - refTime, 1);
 			}
-			GfuiLabelCreate(rmScrHdle, str, GFUI_FONT_MEDIUM_C, x3, y, GFUI_ALIGN_HR_VB, 0);
-			free(str);
+			GfuiLabelCreate(rmScrHdle, timefmt, GFUI_FONT_MEDIUM_C, x3, y, GFUI_ALIGN_HR_VB, 0);
 		} else {
 			dlap = totLaps - laps;
 			if (dlap == 1) {
@@ -306,14 +288,10 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 			GfuiLabelCreate(rmScrHdle, buf, GFUI_FONT_MEDIUM_C, x3, y, GFUI_ALIGN_HR_VB, 0);
 
 		}
-		/*GfuiLabelCreate(rmScrHdle, str, GFUI_FONT_MEDIUM_C,
-				x3, y, GFUI_ALIGN_HR_VB, 0);
-	*/
 
-		str = GfTime2Str(GfParmGetNum(results, path, RE_ATTR_BEST_LAP_TIME, NULL, 0), 0);
-		GfuiLabelCreate(rmScrHdle, str, GFUI_FONT_MEDIUM_C,
+		GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_BEST_LAP_TIME, NULL, 0), 0);
+		GfuiLabelCreate(rmScrHdle, timefmt, GFUI_FONT_MEDIUM_C,
 				x4, y, GFUI_ALIGN_HR_VB, 0);
-		free(str);
 
 		snprintf(buf, BUFSIZE, "%d", laps);
 		GfuiLabelCreate(rmScrHdle, buf, GFUI_FONT_MEDIUM_C,
@@ -330,20 +308,23 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 		snprintf(buf, BUFSIZE, "%d", (int)(GfParmGetNum(results, path, RE_ATTR_NB_PIT_STOPS, NULL, 0)));
 		GfuiLabelCreate(rmScrHdle, buf, GFUI_FONT_MEDIUM_C,
 				x8, y, GFUI_ALIGN_HC_VB, 0);
+
+		GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_PENALTYTIME, NULL, 0), 0);
+		GfuiLabelCreate(rmScrHdle, timefmt, GFUI_FONT_MEDIUM_C, x9, y, GFUI_ALIGN_HR_VB, 0);
+
 		y -= 15;
 	}
 
-
 	if (start > 0) {
-	RmPrevRace.prevHdle = prevHdle;
-	RmPrevRace.info     = info;
-	RmPrevRace.start    = start - MAX_LINES;
-	GfuiGrButtonCreate(rmScrHdle, "data/img/arrow-up.png", "data/img/arrow-up.png",
-			"data/img/arrow-up.png", "data/img/arrow-up-pushed.png",
-			80, 40, GFUI_ALIGN_HL_VB, 1,
-			(void*)&RmPrevRace, rmChgRaceScreen,
-			NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
-	GfuiAddSKey(rmScrHdle, GLUT_KEY_PAGE_UP,   "Previous Results", (void*)&RmPrevRace, rmChgRaceScreen, NULL);
+		RmPrevRace.prevHdle = prevHdle;
+		RmPrevRace.info     = info;
+		RmPrevRace.start    = start - MAX_LINES;
+		GfuiGrButtonCreate(rmScrHdle, "data/img/arrow-up.png", "data/img/arrow-up.png",
+				"data/img/arrow-up.png", "data/img/arrow-up-pushed.png",
+				80, 40, GFUI_ALIGN_HL_VB, 1,
+				(void*)&RmPrevRace, rmChgRaceScreen,
+				NULL, (tfuiCallback)NULL, (tfuiCallback)NULL);
+		GfuiAddSKey(rmScrHdle, GLUT_KEY_PAGE_UP,   "Previous Results", (void*)&RmPrevRace, rmChgRaceScreen, NULL);
 	}
 
 	GfuiButtonCreate(rmScrHdle,
@@ -361,20 +342,6 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 			(tfuiCallback)NULL,
 			(tfuiCallback)NULL);
 
-/*     rmSaveId = GfuiButtonCreate(rmScrHdle, */
-/* 				"Save", */
-/* 				GFUI_FONT_LARGE, */
-/* 				430, */
-/* 				40, */
-/* 				150, */
-/* 				GFUI_ALIGN_HC_VB, */
-/* 				0, */
-/* 				info, */
-/* 				rmSaveRes, */
-/* 				NULL, */
-/* 				(tfuiCallback)NULL, */
-/* 				(tfuiCallback)NULL); */
-
 	if (i < nbCars) {
 		RmNextRace.prevHdle = prevHdle;
 		RmNextRace.info     = info;
@@ -395,8 +362,7 @@ rmRaceResults(void *prevHdle, tRmInfo *info, int start)
 }
 
 
-static void
-rmChgQualifScreen(void *vprc)
+static void rmChgQualifScreen(void *vprc)
 {
 	void *prevScr = rmScrHdle;
 	tRaceCall *prc = (tRaceCall*)vprc;
@@ -406,8 +372,7 @@ rmChgQualifScreen(void *vprc)
 }
 
 
-static void
-rmQualifResults(void *prevHdle, tRmInfo *info, int start)
+static void rmQualifResults(void *prevHdle, tRmInfo *info, int start)
 {
 	void *results = info->results;
 	const char *race = info->_reRaceName;
@@ -417,10 +382,9 @@ rmQualifResults(void *prevHdle, tRmInfo *info, int start)
 	const int BUFSIZE = 1024;
 	char buf[BUFSIZE];
 	char path[BUFSIZE];
-	char *str;
+	const int TIMEFMTSIZE = 256;
+	char timefmt[TIMEFMTSIZE];
 	float fgcolor[4] = {1.0, 0.0, 1.0, 1.0};
-	//int laps, totLaps;
-	//tdble refTime;
 	int nbCars;
 	int offset;
 
@@ -443,16 +407,11 @@ rmQualifResults(void *prevHdle, tRmInfo *info, int start)
 	GfuiLabelCreateEx(rmScrHdle, "Time",      fgcolor, GFUI_FONT_MEDIUM_C, x3, y, GFUI_ALIGN_HR_VB, 0);
 	y -= 20;
 	
-	//snprintf(path, BUFSIZE, "%s/%s/%s", info->track->name, RE_SECT_RESULTS, race);
-	//totLaps = (int)GfParmGetNum(results, path, RE_ATTR_LAPS, NULL, 0);
-	//snprintf(path, BUFSIZE, "%s/%s/%s/%s/%d", info->track->name, RE_SECT_RESULTS, race, RE_SECT_RANK, 1);
-	//refTime = GfParmGetNum(results, path, RE_ATTR_TIME, NULL, 0);
 	snprintf(path, BUFSIZE, "%s/%s/%s/%s", info->track->name, RE_SECT_RESULTS, race, RE_SECT_RANK);
 	nbCars = (int)GfParmGetEltNb(results, path);
 	
 	for (i = start; i < MIN(start + MAX_LINES, nbCars); i++) {
 		snprintf(path, BUFSIZE, "%s/%s/%s/%s/%d", info->track->name, RE_SECT_RESULTS, race, RE_SECT_RANK, i + 1);
-		//laps = (int)GfParmGetNum(results, path, RE_ATTR_LAPS, NULL, 0);
 
 		snprintf(buf, BUFSIZE, "%d", i+1);
 		GfuiLabelCreate(rmScrHdle, buf, GFUI_FONT_MEDIUM_C,
@@ -461,10 +420,9 @@ rmQualifResults(void *prevHdle, tRmInfo *info, int start)
 		GfuiLabelCreate(rmScrHdle, GfParmGetStr(results, path, RE_ATTR_NAME, ""), GFUI_FONT_MEDIUM_C,
 				x2, y, GFUI_ALIGN_HL_VB, 0);
 
-		str = GfTime2Str(GfParmGetNum(results, path, RE_ATTR_BEST_LAP_TIME, NULL, 0), 0);
-		GfuiLabelCreate(rmScrHdle, str, GFUI_FONT_MEDIUM_C,
+		GfTime2Str(timefmt, TIMEFMTSIZE, GfParmGetNum(results, path, RE_ATTR_BEST_LAP_TIME, NULL, 0), 0);
+		GfuiLabelCreate(rmScrHdle, timefmt, GFUI_FONT_MEDIUM_C,
 				x3, y, GFUI_ALIGN_HR_VB, 0);
-		free(str);
 		y -= 15;
 	}
 
@@ -495,20 +453,6 @@ rmQualifResults(void *prevHdle, tRmInfo *info, int start)
 			(tfuiCallback)NULL,
 			(tfuiCallback)NULL);
 
-/*     rmSaveId = GfuiButtonCreate(rmScrHdle, */
-/* 				"Save", */
-/* 				GFUI_FONT_LARGE, */
-/* 				430, */
-/* 				40, */
-/* 				150, */
-/* 				GFUI_ALIGN_HC_VB, */
-/* 				0, */
-/* 				info, */
-/* 				rmSaveRes, */
-/* 				NULL, */
-/* 				(tfuiCallback)NULL, */
-/* 				(tfuiCallback)NULL); */
-
 	if (i < nbCars) {
 		RmNextRace.prevHdle = prevHdle;
 		RmNextRace.info     = info;
@@ -529,8 +473,7 @@ rmQualifResults(void *prevHdle, tRmInfo *info, int start)
 }
 
 
-static void
-rmChgStandingScreen(void *vprc)
+static void rmChgStandingScreen(void *vprc)
 {
 	void *prevScr = rmScrHdle;
 	tRaceCall *prc = (tRaceCall*)vprc;
@@ -540,8 +483,7 @@ rmChgStandingScreen(void *vprc)
 }
 
 
-static void
-rmShowStandings(void *prevHdle, tRmInfo *info, int start)
+static void rmShowStandings(void *prevHdle, tRmInfo *info, int start)
 {
 	int i;
 	int x1, x2, x3;
@@ -650,8 +592,12 @@ rmShowStandings(void *prevHdle, tRmInfo *info, int start)
 }
 
 
-void
-RmShowResults(void *prevHdle, tRmInfo *info)
+/** @brief Display results
+ *  @ingroup racemantools
+ *  @param[in] prevHdle Handle to previous result screen (used if the results require more than one screen)
+ *  @param[in] info tRmInfo.results carries the result parameter set handle
+ */
+void RmShowResults(void *prevHdle, tRmInfo *info)
 {
 	switch (info->s->_raceType) {
 		case RM_TYPE_PRACTICE:
@@ -669,8 +615,12 @@ RmShowResults(void *prevHdle, tRmInfo *info)
 }
 
 
-void
-RmShowStandings(void *prevHdle, tRmInfo *info)
+/** @brief Display standings
+ *  @ingroup racemantools
+ *  @param[in] prevHdle Handle to previous standings screen (used if the standings require more than one screen)
+ *  @param[in] info tRmInfo.results carries the result parameter set handle containing the standings
+ */
+void RmShowStandings(void *prevHdle, tRmInfo *info)
 {
     rmShowStandings(prevHdle, info, 0);
 }
