@@ -2,9 +2,9 @@
                            screen.cpp -- screen init
                              -------------------
     created              : Fri Aug 13 22:29:56 CEST 1999
-    copyright            : (C) 1999, 2004 by Eric Espie, Bernhard Wymann
+    copyright            : (C) 1999, 2014 by Eric Espie, Bernhard Wymann
     email                : torcs@free.fr
-    version              : $Id: screen.cpp,v 1.23.2.5 2011/12/29 16:14:21 berniw Exp $
+    version              : $Id: screen.cpp,v 1.23.2.11 2014/05/23 08:38:32 berniw Exp $
 ***************************************************************************/
 
 /***************************************************************************
@@ -18,9 +18,8 @@
 
 /** @file
     Screen management.
-    @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: screen.cpp,v 1.23.2.5 2011/12/29 16:14:21 berniw Exp $
-    @ingroup	screen
+    @author bernhard Wymann, Eric Espie
+    @version $Id: screen.cpp,v 1.23.2.11 2014/05/23 08:38:32 berniw Exp $
 */
 
 #include <stdio.h>
@@ -102,8 +101,6 @@ static int	DepthLabelId;
 static int	ModeLabelId;
 static int VInitLabelId;
 
-static void	*paramHdle;
-
 static float LabelColor[] = {1.0, 0.0, 1.0, 1.0};
 
 
@@ -460,10 +457,15 @@ static void
 saveParams(void)
 {
 	int x, y, bpp;
-
+	
 	sscanf(Res[curRes], "%dx%d", &x, &y);
 	sscanf(Depthlist[curDepth], "%d", &bpp);
 
+	const int BUFSIZE = 1024;
+	char buf[BUFSIZE];
+	snprintf(buf, BUFSIZE, "%s%s", GetLocalDir(), GFSCR_CONF_FILE);
+	void *paramHdle = GfParmReadFile(buf, GFPARM_RMODE_STD | GFPARM_RMODE_CREAT);
+
 	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_X, (char*)NULL, x);
 	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_Y, (char*)NULL, y);
 	GfParmSetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_WIN_X, (char*)NULL, x);
@@ -479,6 +481,7 @@ saveParams(void)
 		GfParmSetStr(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_FSCR, "no");
 	}
 	GfParmWriteFile(NULL, paramHdle, "Screen");
+	GfParmReleaseHandle(paramHdle);
 }
 
 
@@ -499,7 +502,7 @@ GfScrReinit(void * /* dummy */)
     saveParams();
 
 #ifdef WIN32
-	snprintf(cmd, CMDSIZE, "%swtorcs.exe", GetLocalDir());
+	snprintf(cmd, CMDSIZE, "%swtorcs.exe", GetLibDir());
     int i;
 	for (i = 0; i < CMDSIZE && cmd[i] != NULL; i++) {
 		if (cmd[i] == '/') {
@@ -508,7 +511,7 @@ GfScrReinit(void * /* dummy */)
 	}
 	
 	char cmdarg[CMDSIZE];
-	snprintf(cmdarg, CMDSIZE, "\"%swtorcs.exe\"", GetLocalDir());
+	snprintf(cmdarg, CMDSIZE, "\"%swtorcs.exe\"", GetLibDir());
 	for (i = 0; i < CMDSIZE && cmdarg[i] != NULL; i++) {
 		if (cmdarg[i] == '/') {
 			cmdarg[i] = '\\';
@@ -668,6 +671,9 @@ initFromConf(void)
 	const int BUFSIZE = 1024;
 	char buf[BUFSIZE];
 
+	snprintf(buf, BUFSIZE, "%s%s", GetLocalDir(), GFSCR_CONF_FILE);
+	void *paramHdle = GfParmReadFile(buf, GFPARM_RMODE_STD | GFPARM_RMODE_CREAT);
+
 	x = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_X, NULL, 640);
 	y = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_Y, NULL, 480);
 
@@ -704,6 +710,7 @@ initFromConf(void)
 	}
 
 	curMaxFreq = (int)GfParmGetNum(paramHdle, GFSCR_SECT_PROP, GFSCR_ATT_MAXREFRESH, NULL, curMaxFreq);
+	GfParmReleaseHandle(paramHdle);
 }
 
 #ifdef WIN32
@@ -737,16 +744,12 @@ void *
 GfScrMenuInit(void *precMenu)
 {
     int		y, x1, x2;
-	const int BUFSIZE = 1024;
-	char buf[BUFSIZE];
 	
 #ifndef WIN32
 	const int yoffset1 = 30, yoffset2 = 60;
 #else // WIN32
 	const int yoffset1 = 30, yoffset2 = 40;
 #endif // WIN32
-    snprintf(buf, BUFSIZE, "%s%s", GetLocalDir(), GFSCR_CONF_FILE);
-    paramHdle = GfParmReadFile(buf, GFPARM_RMODE_STD | GFPARM_RMODE_CREAT);
 
     if (scrHandle) return scrHandle;
 
