@@ -2,9 +2,9 @@
 
     file                 : grboard.cpp
     created              : Thu Aug 17 23:52:20 CEST 2000
-    copyright            : (C) 2000 by Eric Espie
+    copyright            : (C) 2000-2014 by Eric Espie, Bernhard Wymann
     email                : torcs@free.fr
-    version              : $Id: grboard.cpp,v 1.27.2.4 2012/02/09 22:36:26 berniw Exp $
+    version              : $Id: grboard.cpp,v 1.27.2.8 2014/03/04 00:37:17 berniw Exp $
 
  ***************************************************************************/
 
@@ -295,8 +295,8 @@ cGrBoard::grDispCarBoard1(tCarElt *car, tSituation *s)
 	glColor4f(0.1, 0.1, 0.1, 0.8);
 	glVertex2f(x-5, y + dy);
 	glVertex2f(x+dx+5, y + dy);
-	glVertex2f(x+dx+5, y-5 - dy2 * 8 /* lines */);
-	glVertex2f(x-5, y-5 - dy2 * 8 /* lines */);
+	glVertex2f(x+dx+5, y-5 - dy2 * 9 /* lines */);
+	glVertex2f(x-5, y-5 - dy2 * 9 /* lines */);
 	glEnd();
 	glDisable(GL_BLEND);
 	
@@ -337,8 +337,12 @@ cGrBoard::grDispCarBoard1(tCarElt *car, tSituation *s)
 	y -= dy;
 	
 	GfuiPrintString("Curr:", clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
+	if (!car->_commitBestLapTime) {
+		clr = grRed;
+	}
 	grWriteTime(clr, GFUI_FONT_SMALL_C, x2, y, car->_curLapTime, 0);
 	y -= dy;
+	clr = grWhite;
 	
 	GfuiPrintString("Last:", clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
 	grWriteTime(clr, GFUI_FONT_SMALL_C, x2, y, car->_lastLapTime, 0);
@@ -348,10 +352,13 @@ cGrBoard::grDispCarBoard1(tCarElt *car, tSituation *s)
 	grWriteTime(clr, GFUI_FONT_SMALL_C, x2, y, car->_bestLapTime, 0);
 	y -= dy;
 	
+	GfuiPrintString("Penalty:", clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
+	grWriteTime(clr, GFUI_FONT_SMALL_C, x2, y, car->_penaltyTime, 0);
+	y -= dy;
+
 	GfuiPrintString("Top Speed:", clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
 	snprintf(buf, BUFSIZE, "%d", (int)(car->_topSpeed * 3.6));
 	GfuiPrintString(buf, clr, GFUI_FONT_SMALL_C, x2, y, GFUI_ALIGN_HR_VB);
-	y -= dy;
 }
 
 void
@@ -375,7 +382,7 @@ cGrBoard::grDispCarBoard2(tCarElt *car, tSituation *s)
 	snprintf(buf, BUFSIZE, "%d/%d - %s", car->_pos, s->_ncars, car->_name);
 	dx = GfuiFontWidth(GFUI_FONT_MEDIUM_C, buf);
 	dx = MAX(dx, (x3-x));
-	lines = 6;
+	lines = 7;
 	for (i = 0; i < 4; i++) {
 		if (car->ctrl.msg[i]) {
 			lines++;
@@ -421,9 +428,17 @@ cGrBoard::grDispCarBoard2(tCarElt *car, tSituation *s)
 	y -= dy;
 	
 	GfuiPrintString("Time:", clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
+	if (!car->_commitBestLapTime) {
+		clr = grRed;
+	}
 	grWriteTime(clr, GFUI_FONT_SMALL_C, x2, y, car->_curLapTime, 0);    
 	y -= dy;
+	clr = grWhite;
 	
+	GfuiPrintString("Penalty:", clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
+	grWriteTime(clr, GFUI_FONT_SMALL_C, x2, y, car->_penaltyTime, 0);
+	y -= dy;
+
 	if (car->_pos != 1) {
 		snprintf(buf, BUFSIZE, "<- %s", s->cars[car->_pos - 2]->_name);
 		GfuiPrintString(buf, clr, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
@@ -451,6 +466,7 @@ cGrBoard::grDispCarBoard2(tCarElt *car, tSituation *s)
 		GfuiPrintString("       --:--", clr, GFUI_FONT_SMALL_C, x3, y, GFUI_ALIGN_HR_VB);
 	}
 	y -= dy;
+
 	for (i = 0; i < 4; i++) {
 		if (car->ctrl.msg[i]) {
 			GfuiPrintString(car->ctrl.msg[i], car->ctrl.msgColor, GFUI_FONT_SMALL_C, x, y, GFUI_ALIGN_HL_VB);
@@ -682,19 +698,6 @@ cGrBoard::grDispLeaderBoard(tCarElt *car, tSituation *s)
 	}
 }
 
-class myLoaderOptions : public ssgLoaderOptions
-{
-public:
-  virtual void makeModelPath ( char* path, const char *fname ) const
-  {
-    ulFindFile ( path, model_dir, fname, NULL ) ;
-  }
-
-  virtual void makeTexturePath ( char* path, const char *fname ) const
-  {
-    ulFindFile ( path, texture_dir, fname, NULL ) ;
-  }
-} ;
 
 void
 cGrBoard::grDispCounterBoard2(tCarElt *car)
@@ -813,7 +816,12 @@ cGrBoard::grDispArcade(tCarElt *car, tSituation *s)
 	dy = GfuiFontHeight(GFUI_FONT_LARGE_C);
 	y -= dy;
 	GfuiPrintString("Time:", grDefaultClr, GFUI_FONT_LARGE_C, x, y, GFUI_ALIGN_HL_VB);
-	grWriteTime(grDefaultClr, GFUI_FONT_LARGE_C, x + 150, y, car->_curLapTime, 0);
+	if (!car->_commitBestLapTime) {
+		clr = grRed;
+	} else {
+		clr = grDefaultClr;
+	}
+	grWriteTime(clr, GFUI_FONT_LARGE_C, x + 150, y, car->_curLapTime, 0);
 
 	y -= dy;
 	GfuiPrintString("Best:", grDefaultClr, GFUI_FONT_LARGE_C, x, y, GFUI_ALIGN_HL_VB);
@@ -867,11 +875,6 @@ void cGrBoard::refreshBoard(tSituation *s, float Fps, int forceArcade, tCarElt *
 }
 
 
-// TODO: clean solution for cleanup.
-static ssgSimpleState* cleanup[1024];
-static int nstate = 0;
-
-
 void grInitBoardCar(tCarElt *car)
 {
 	const int BUFSIZE=1024;
@@ -899,9 +902,8 @@ void grInitBoardCar(tCarElt *car)
 	snprintf(buf, BUFSIZE, "drivers/%s/%d;drivers/%s;cars/%s;data/textures", car->_modName, car->_driverIndex, car->_modName, car->_carName);
 	grFilePath = strdup(buf);
 	curInst->texture = (ssgSimpleState*)grSsgLoadTexState(param);
+	curInst->texture->ref();
 	free(grFilePath);
-	cleanup[nstate] = curInst->texture;
-	nstate++;
 	
 	/* Load the intrument placement */
 	xSz = GfParmGetNum(handle, SECT_GROBJECTS, PRM_TACHO_XSZ, (char*)NULL, 128);
@@ -956,9 +958,8 @@ void grInitBoardCar(tCarElt *car)
 	snprintf(buf, BUFSIZE, "drivers/%s/%d;drivers/%s;cars/%s;data/textures", car->_modName, car->_driverIndex, car->_modName, car->_carName);
 	grFilePath = strdup(buf);
 	curInst->texture = (ssgSimpleState*)grSsgLoadTexState(param);
+	curInst->texture->ref();
 	free(grFilePath);
-	cleanup[nstate] = curInst->texture;
-	nstate++;
 	
 	/* Load the intrument placement */
 	xSz = GfParmGetNum(handle, SECT_GROBJECTS, PRM_SPEEDO_XSZ, (char*)NULL, 128);
@@ -1011,14 +1012,13 @@ void grInitBoardCar(tCarElt *car)
 
 void grShutdownBoardCar(void)
 {
-	/*int i;
-	for (i = 0; i < nstate; i++) {
-		printf("%d\n", i);
-		if (cleanup[i]->getRef() > 0) {
-			ssgDeRefDelete(cleanup[i]);
-		} else {
-			delete cleanup[i];
-		}
+	int i;
+	for (i = 0; i < grNbCars; i++) {
+		ssgDeRefDelete(grCarInfo[i].instrument[0].texture);	
+		ssgDeRefDelete(grCarInfo[i].instrument[1].texture);
+		glDeleteLists(grCarInfo[i].instrument[0].needleList, 1);
+		glDeleteLists(grCarInfo[i].instrument[1].needleList, 1);
+		glDeleteLists(grCarInfo[i].instrument[0].CounterList, 1);
+		glDeleteLists(grCarInfo[i].instrument[1].CounterList, 1);
 	}
-	nstate = 0;*/
 }
