@@ -2,8 +2,8 @@
 
     file                 : grsmoke.cpp
     created              : Fri Mar 22 23:17:54 CET 2002
-    copyright            : (C) 2001 by Christophe Guionneau
-    version              : $Id: grsmoke.cpp,v 1.27.2.3 2012/02/09 22:36:26 berniw Exp $
+    copyright            : (C) 2001-2014 by Christophe Guionneau, Bernhard Wymann
+    version              : $Id: grsmoke.cpp,v 1.27.2.5 2014/02/10 11:06:17 berniw Exp $
 
 ***************************************************************************/
 
@@ -109,6 +109,8 @@ void grInitSmoke(int index)
 			mst->disable(GL_CULL_FACE);
 			mst->setTranslucent();
 			mst->setColourMaterial(GL_AMBIENT_AND_DIFFUSE);
+			// Reference material to keep it,must be delete manually later
+			mst->ref();
 		}
     }
 
@@ -121,6 +123,8 @@ void grInitSmoke(int index)
 			mstf0->disable(GL_CULL_FACE);
 			mstf0->setTranslucent();
 			mstf0->setColourMaterial(GL_AMBIENT_AND_DIFFUSE);
+			// Reference material to keep it,must be delete manually later
+			mstf0->ref();
 		}
     }
 
@@ -133,6 +137,8 @@ void grInitSmoke(int index)
 			mstf1->disable(GL_CULL_FACE);
 			mstf1->setTranslucent();
 			mstf1->setColourMaterial(GL_AMBIENT_AND_DIFFUSE);
+			// Reference material to keep it,must be delete manually later
+			mstf1->ref();
 		}
     }
 }
@@ -220,17 +226,14 @@ void grAddSmoke(tCarElt *car, double t)
 	ssgVertexArray *shd_vtx ;
 	tgrCarInstrument *curInst;
 	tdble val;
-	tdble spd2;
 	int index;
 
 	if (!grSmokeMaxNumber) {
 		return;
 	}
 
-	spd2 = car->_speed_x * car->_speed_x + car->_speed_y * car->_speed_y;
-
 	for (i = 0; i < 4; i++) {
-		if (spd2 > 0.001f) {
+		if (car->pub.speed > 0.03f) {	// 0.03 -> sqrt(0.001)
 			if (smokeManager->number < grSmokeMaxNumber) {
 				if ((t - timeSmoke[car->index*4+i]) < grSmokeDeltaT) {
 					continue;
@@ -296,7 +299,7 @@ void grAddSmoke(tCarElt *car, double t)
 				}
 
 				smoke_life_coefficient = smoke_life_coefficient * (1.0f - urandom()*urandom());
-				tdble spd_fx=tanh(0.001f*car->_reaction[i])*smoke_speed_coefficient*sqrt(spd2);
+				tdble spd_fx=tanh(0.001f*car->_reaction[i])*smoke_speed_coefficient*car->pub.speed;
 				if (car->_skid[i] + 0.025f*urandom()*spd_fx>urandom() + threshold) {// instead of 0.3, to randomize
 
 					float init_speed_z = 0.1f;
@@ -333,7 +336,7 @@ void grAddSmoke(tCarElt *car, double t)
 
 					//printf("%f\n", car->_reaction[i]);
 					tmp->smoke->max_life = grSmokeLife *
-						(car->_skid[i]*sqrt(spd2)+urandom()*spd_fx)/ smoke_life_coefficient;
+						(car->_skid[i]*car->pub.speed+urandom()*spd_fx)/ smoke_life_coefficient;
 					for (int c = 0; c < 3; c++) {
 						tmp->smoke->cur_col[c] = cur_clr[c];
 					}
@@ -363,7 +366,7 @@ void grAddSmoke(tCarElt *car, double t)
 		}
     }
 
-    if (car->_exhaustNb && (spd2 > 10.0)) {
+    if (car->_exhaustNb && (car->pub.speed > 3.0f)) {
 		if (smokeManager->number < grSmokeMaxNumber) {
 			index = car->index;	/* current car's index */
 			if ((t - timeFire[index]) > grFireDeltaT) {
@@ -452,6 +455,21 @@ void grShutdownSmoke ()
 		timeSmoke = NULL;
 		timeFire=NULL;
     }
+
+	if (mst != NULL) {
+		ssgDeRefDelete(mst);
+		mst = NULL;
+	}
+
+	if (mstf0 != NULL) {
+		ssgDeRefDelete(mstf0);
+		mstf0 = NULL;
+	}
+
+	if (mstf1 != NULL) {
+		ssgDeRefDelete(mstf1);
+		mstf1 = NULL;
+	}
 }
 
 void ssgVtxTableSmoke::copy_from ( ssgVtxTableSmoke *src, int clone_flags )
